"""
DhravyaAPI Wrapper
~~~~~~~~~~~~~~~~~~

A basic asynchronous wrapper for the [DhravyaAPI](https://api.dhravya.me).
"""

__title__ = "dhravyapy"
__author__ = "27Saumya"
__license__ = "MIT"
__version__ = "0.0.9"

from .fun import *
from .image import *
from .info import *
from .misc import *
from typing import Dict, Union


async def stats() -> Dict[str, Union[str, int]]:
    """
    :class:`dict`: Gets the stats of the API.
    """
    response = await HTTPClient().get("stats")

    if response.status == 200:
        json = await response.json()
        return json["data"]

    elif response.status == 422:
        raise ValidationError("Recieved an invalid input")

    else:
        raise HTTPException(f"HTTP Error: {response.status}")



"""
MIT License

Copyright (c) 2022 27Saumya

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from .http import HTTPClient
from typing import *
from .errors import *
from .assets import Animal, GeneralImage


class Image:
    """
    The class to get Image data from the API
    """

    @classmethod
    async def cat(cls) -> Animal:
        """
        :class:`GeneralImage` Gets a random cat picture.
        """
        response = await HTTPClient().get("cat")

        if response.status == 200:
            return Animal(await response.read())

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def dog(cls) -> Animal:
        """
        :class:`GeneralImage` Gets a random dog picture.
        """
        response = await HTTPClient().get("dog")

        if response.status == 200:
            return Animal(await response.read())

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def fox(cls) -> Animal:
        """
        :class:`GeneralImage` Gets a random fox picture.
        """
        response = await HTTPClient().get("fox")

        if response.status == 200:
            return Animal(await response.read())

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def create_meme(cls, top: str, bottom: str, image: Optional[str] = None) -> GeneralImage:
        """
        :class:`GeneralImage` Creates a meme image.
        """
        response = await HTTPClient().get(f"meme?top={top}&bottom={bottom}&image={image}")

        if response.status == 200:
            return GeneralImage(await response.read())

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def mealsome(cls, me: str, alsome: str) -> GeneralImage:
        """
        :class:`GeneralImage` Creates a me - also me meme image.
        """
        response = await HTTPClient().get(f"mealsome?me={me}&alsome={alsome}")

        if response.status == 200:
            return GeneralImage(await response.read())

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def itsretarded(cls, text: str) -> GeneralImage:
        """
        :class:`GeneralImage` Creates a retarded meme image.
        """
        response = await HTTPClient().get(f"itsretarded?text={text}")

        if response.status == 200:
            return GeneralImage(await response.read())

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def headache(cls, text: str) -> GeneralImage:
        """
        :class:`GeneralImage` Creates a headache meme image.
        """
        response = await HTTPClient().get(f"headache?text={text}")

        if response.status == 200:
            return GeneralImage(await response.read())

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def classnote(cls, text: str) -> GeneralImage:
        """
        :class:`GeneralImage` Creates a classnote meme image.
        """
        response = await HTTPClient().get(f"classnote?text={text}")

        if response.status == 200:
            return GeneralImage(await response.read())

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def nutbutton(cls, text: str) -> GeneralImage:
        """
        :class:`GeneralImage` Creates a nutbutton meme image.
        """
        response = await HTTPClient().get(f"nutbutton?text={text}")

        if response.status == 200:
            return GeneralImage(await response.read())

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def pills(cls, text: str) -> GeneralImage:
        """
        :class:`GeneralImage` Creates a pills meme image.
        """
        response = await HTTPClient().get(f"pills?text={text}")

        if response.status == 200:
            return GeneralImage(await response.read())

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def balloon(cls, text: str, person: str, stopper: str) -> GeneralImage:
        """
        :class:`GeneralImage` Creates a balloon meme image.
        """
        response = await HTTPClient().get(f"balloon?text={text}&person={person}&stopper={stopper}")

        if response.status == 200:
            return GeneralImage(await response.read())

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def waifu(cls) -> GeneralImage:
        """
        :class:`GeneralImage` Gets a waifu image.
        """
        response = await HTTPClient().get("waifu")

        if response.status == 200:
            return GeneralImage(await response.read())

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def qrcode(cls, query: str, drawer: Optional[int] = 1, mask: Optional[int] = 1) -> GeneralImage:
        """
        :class:`GeneralImage` Gets a qrcode image.
        """
        response = await HTTPClient().get(f"qrcode?query={query}&drawer={drawer}&mask={mask}")

        if response.status == 200:
            return GeneralImage(await response.read())

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")



"""
MIT License

Copyright (c) 2022 27Saumya

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from .http import HTTPClient
from typing import *
from .errors import *
from .assets import RandomUserInfo


class Misc:
    """
    The class to get Miscellaneous data from the API
    """

    @classmethod
    async def compliment(cls, simple: Optional[bool] = False) -> str:
        """
        :class:`str`: Gets a random compliment.

        Parameters
        ----------
        simple : Wheter the compliment should be simple. Optional[:class:`bool`]
        """
        response = await HTTPClient().get(f"compliment?simple={simple}")

        if response.status == 200:
            json = await response.json()
            return json["data"]["Compliment"]

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def topic(cls, simple: Optional[bool] = False) -> str:
        """
        :class:`str`: Gets a random topic.

        Parameters
        ----------
        simple : Wheter the topic should be simple. Optional[:class:`bool`]
        """
        response = await HTTPClient().get(f"topic?simple={simple}")

        if response.status == 200:
            json = await response.json()
            return json["data"]["Topic"]

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def translate(
        cls,
        text: str,
        from_lang: str,
        to_lang: Optional[str] = "en",
        simple: Optional[bool] = False,
    ) -> str:
        """
        :class:`str`: Translates text from one language to another.

        Parameters
        ----------
        text : The text to translate. :class:`str`
        from_lang : The language to translate from. :class:`str`
        to_lang : The language to translate to. Optional[:class:`str`]
        simple : Wheter the translation should be simple. Optional[:class:`bool`]
        """
        response = await HTTPClient().get(f"translate?text={text}&from={from_lang}&to={to_lang}&simple={simple}")

        if response.status == 200:
            json = await response.json()
            return json["data"]  # Don't know what it returns.. so returned dict for now

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def ascii(cls, text: str, font: Optional[str] = None):
        """
        :class:`str`: Gets an ASCII art of the text.

        Parameters
        ----------
        text : The text to get an ASCII art of. :class:`str`
        font : The font to use. Optional[:class:`str`]
        """
        response = await HTTPClient().get(f"ascii?text={text}&font={font}")

        if response.status == 200:
            json = await response.json()
            return json["data"]

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def randomuser(cls) -> RandomUserInfo:
        """
        :class:`RandomUserInfo`: Gets a random user.
        """
        response = await HTTPClient().get("randomuser")

        if response.status == 200:
            json = await response.json()
            return RandomUserInfo(json)

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def autofill(cls, query: str) -> List[str]:
        """
        :class:`List[str]`: Gets autofill suggestions for the query.

        Parameters
        ----------
        query : The query to get autofill suggestions for. :class:`str`
        """
        response = await HTTPClient().get(f"autofill?query={query}")

        if response.status == 200:
            json = await response.json()
            return json["data"]

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")



"""
MIT License

Copyright (c) 2022 27Saumya

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from typing import *
from io import BytesIO
import aiofiles

from .http import HTTPClient
from .errors import HTTPException
import dhravyapy


class Meme:
    """
    The class to get meme related data from the API
    """

    def __init__(self, json: dict):
        self.json = json

    async def save(self, path: str) -> None:
        """
        Saves the meme as an image.

        Parameters
        ----------
        path : Filename and the Path to save the image :class:`str`

        Example
        -------
        >>> meme = await dhravyapy.Fun().meme("random")
        >>> await meme.save(f"meme.{meme.file_extension}")
        """
        r = await HTTPClient()._get(self.url)
        if r.status == 200:
            f = await aiofiles.open(path, "wb")
            await f.write(await r.read())
            await f.close()
        else:
            raise HTTPException(f"HTTP Error: {r.status}")

    async def get_bytes(self) -> BytesIO:
        """
        Returns the meme as a BytesIO object

        Example
        -------
        >>> meme = await dhravyapy.Fun().meme("random")
        >>> meme_bytes = await meme.get_bytes()
        """
        r = await HTTPClient()._get(self.url)
        if r.status == 200:
            return BytesIO(await r.read())
        else:
            raise HTTPException(f"HTTP Error: {r.status}")

    @property
    def url(self) -> str:
        """
        :class:`str`: The URL of the meme.
        """
        return self.json["data"]["url"]

    @property
    def id(self) -> str:
        """
        :class:`str`: The ID of the meme.
        """
        return self.json["data"]["id"]

    @property
    def subreddit(self) -> str:
        """
        :class:`str`: The subreddit of the meme.
        """
        return self.json["data"]["subreddit"]

    @property
    def title(self) -> str:
        """
        :class:`str`: The title of the meme.
        """
        return self.json["data"]["title"]

    @property
    def score(self) -> int:
        """
        :class:`int`: The score of the meme.
        """
        return self.json["data"]["score"]

    @property
    def selftext(self) -> str:
        """
        :class:`str`: The selftext of the meme.
        """
        return self.json["data"]["selftext"]

    @property
    def is_nsfw(self) -> bool:
        """
        :class:`bool`: Whether the meme is NSFW.
        """
        return self.json["data"]["is_nsfw"]

    @property
    def dict(self) -> dict:
        """
        :class:`dict`: The meme's json in a dictionary.
        """
        return self.json["data"]

    @property
    def file_extension(self) -> str:
        """
        :class:`str`: The file extension of the meme.
        """
        return self.url.split(".")[-1]


class TruthOrDare:
    """
    The class to return the data from the "truthordare" endpoint.
    """

    def __init__(self, json: dict):
        self.json = json

    @property
    def truth(self) -> str:
        """
        :class:`str`: The truth.
        """
        return self.json["data"]["Truth"]

    @property
    def dare(self) -> str:
        """
        :class:`str`: The dare.
        """
        return self.json["data"]["Dare"]


class Trivia:
    """
    The class to return the data from the "trivia" endpoint.
    """

    def __init__(self, json: dict):
        self.json = json

    @property
    def question(self) -> str:
        """
        :class:`str`: The question.
        """
        return self.json["data"]["Question"]

    @property
    def answer(self) -> str:
        """
        :class:`str`: The answer.
        """
        return self.json["data"]["Answer"]


class SongInfo:
    """
    The class to return the data from the "songinfo" endpoint.
    """

    def __init__(self, json: dict):
        self.json = json

    @property
    def title(self) -> str:
        """
        :class:`str`: The title of the song.
        """
        return self.json["response"]["result"]["full_title"]

    @property
    def artist(self) -> str:
        """
        :class:`str`: The artist of the song.
        """
        return self.json["response"]["result"]["artist_names"]

    @property
    def thumnail_image_url(self) -> str:
        """
        :class:`str`: The thumbnail image url of the song.
        """
        return self.json["response"]["result"]["header_image_thumbnail_url"]

    @property
    def image_url(self) -> str:
        """
        :class:`str`: The image url of the song.
        """
        return self.json["response"]["result"]["header_image_url"]

    async def lyrics(self) -> str:
        """
        :class:`str`: The lyrics of the song.
        """
        return await dhravyapy.info.Info.lyrics(self.json["response"]["result"]["full_title"])

    @property
    def dict(self) -> dict:
        """
        :class:`dict`: The song's json in a dictionary.
        """
        return self.json["response"]


class BoredInfo:
    """
    The class to return the data from the "bored" endpoint.
    """

    def __init__(self, json: dict):
        self.json = json

    @property
    def activity(self) -> str:
        """
        :class:`str`: The activity.
        """
        return self.json["data"]["activity"]

    @property
    def type(self) -> str:
        """
        :class:`str`: The type of the activity.
        """
        return self.json["data"]["type"]

    @property
    def participants(self) -> int:
        """
        :class:`str`: The participants of the activity.
        """
        return self.json["data"]["participants"]

    @property
    def price(self) -> int:
        """
        :class:`str`: The price of the activity.
        """
        return self.json["data"]["price"]

    @property
    def link(self) -> str:
        """
        :class:`str`: The link of the activity.
        """
        return self.json["data"]["link"]

    @property
    def accessibility(self) -> Union[float, int]:
        """
        :class:`str`: The accessibility of the activity.
        """
        return self.json["data"]["access"]

    @property
    def dict(self) -> dict:
        """
        :class:`dict`: The activity's json in a dictionary.
        """
        return self.json["data"]


class RandomUserInfo:
    """
    The class to return the data from the "randomuser" endpoint.
    """

    def __init__(self, json: dict):
        self.json = json

    @property
    def name(self) -> str:
        """
        :class:`str`: The name of the user.
        """
        return self.json["data"]["name"]["first"]

    @property
    def lastname(self) -> str:
        """
        :class:`str`: The lastname of the user.
        """
        return self.json["data"]["name"]["last"]

    @property
    def full_name(self) -> str:
        """
        :class:`str`: The full name of the user.
        """
        return self.json["data"]["name"]["first"] + " " + self.json["data"]["name"]["last"]

    @property
    def email(self) -> str:
        """
        :class:`str`: The email of the user.
        """
        return self.json["data"]["email"]

    @property
    def gender(self) -> str:
        """
        :class:`str`: The gender of the user.
        """
        return self.json["data"]["gender"]

    @property
    def location(self) -> Dict[str, Union[int, str]]:
        """
        Dict[Union[:class:`int`, :class:`str`]]`: The location of the user in a dictionary
        """
        return self.json["location"]

    @property
    def timezone_offset(self) -> str:
        """
        :class:`str`: The timezone offset of the user.
        """
        return self.json["data"]["location"]["timezone"]["offset"]

    @property
    def timezone_region(self) -> str:
        """
        :class:`str`: The timezone region of the user.
        """
        return self.json["data"]["location"]["timezone"]["description"]

    @property
    def login(self) -> Dict[str, Union[str, int]]:
        """
        Dict[:class:`str`] The login of the user in a dictionary
        """
        return self.json["data"]["login"]

    @property
    def dob(self) -> str:
        """
        :class:`str`: The date of birth of the user.
        """
        return self.json["data"]["dob"]["date"]

    @property
    def age(self) -> int:
        """
        :class:`int`: The age of the user.
        """
        return self.json["data"]["dob"]["age"]

    @property
    def registered(self) -> str:
        """
        :class:`str`: The registered date of the user.
        """
        return self.json["data"]["registered"]["date"]

    @property
    def phone(self) -> str:
        """
        :class:`str`: The phone number of the user.
        """
        return self.json["data"]["phone"]

    @property
    def cell(self) -> str:
        """
        :class:`str`: The cell phone number of the user.
        """
        return self.json["data"]["cell"]

    @property
    def id(self) -> Dict[str, Union[int, str]]:
        """
        Dict[:class:`str`]: The id of the user.
        """
        return self.json["data"]["id"]["value"]

    @property
    def large_picture(self) -> str:
        """
        :class:`str`: The large picture of the user.
        """
        return self.json["data"]["picture"]["large"]

    @property
    def medium_picture(self) -> str:
        """
        :class:`str`: The medium picture of the user.
        """
        return self.json["data"]["picture"]["medium"]

    @property
    def thumbnail(self) -> str:
        """
        :class:`str`: The thumbnail picture of the user.
        """
        return self.json["data"]["picture"]["thumbnail"]

    @property
    def dict(self) -> dict:
        """
        :class:`dict`: The user's json in a dictionary.
        """
        return self.json["data"]


class Animal:
    """
    The class to return the data from the "animal" endpoint.
    """

    def __init__(self, _bytes: Union[bytes, str]):
        self.bytes = _bytes

    async def save(self, path: str) -> None:
        """
        Save the image of the animal.

        Parameters
        ----------
        path : Filename and the Path to save the image :class:`str`

        Example
        -------
        >>> dog = await dhravyapy.Animal().dog()
        >>> await dog.save("dog.png")
        """
        f = await aiofiles.open(path, "wb")
        await f.write(self.bytes)
        await f.close()

    async def get_bytes(self) -> bytes:
        """
        Get the image of the animal in bytes.

        Example
        -------
        >>> dog = await dhravyapy.Animal().dog()
        >>> await dog.get_bytes()
        """
        return BytesIO(self.bytes)


class GeneralImage:
    """
    The class to return the data from images in general endpoints.
    """

    def __init__(self, _bytes: Union[bytes, str]):
        self.bytes = _bytes

    async def save(self, path: str) -> None:
        """
        Save the image.

        Parameters
        ----------
        path : Filename and the Path to save the image :class:`str`

        Example
        -------
        >>> waifu = await dhravyapy.Image().waifu()
        >>> await waifu.save("waifu.png")
        """
        f = await aiofiles.open(path, "wb")
        await f.write(self.bytes)
        await f.close()

    async def get_bytes(self) -> bytes:
        """
        Get the image in bytes.

        Example
        -------
        >>> waifu = await dhravyapy.Image().waifu()
        >>> await waifu.get_bytes()
        """
        return BytesIO(self.bytes)



"""
MIT License

Copyright (c) 2022 27Saumya

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from .http import HTTPClient
from typing import *
from .errors import *
from .assets import Trivia, SongInfo


class Info:
    """
    The class to get information related data from the API
    """

    @classmethod
    async def fact(cls, simple: Optional[bool] = False) -> str:
        """
        :class:`str`: Gets a random fact.

        Parameters
        ----------
        simple : Wheter the fact should be simple. Optional[:class:`bool`]
        """
        response = await HTTPClient().get(f"fact?simple={simple}")

        if response.status == 200:
            json = await response.json()
            return json["data"]["Fact"]

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def trivia(cls, simple: Optional[bool] = False) -> Trivia:
        """
        :class:`str`: Gets a random trivia question.

        Parameters
        ----------
        simple : Wheter the trivia question should be simple. Optional[:class:`bool`]
        """
        response = await HTTPClient().get(f"trivia?simple={simple}")

        if response.status == 200:
            json = await response.json()
            return Trivia(json)

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def lyrics(cls, song: str, simple: Optional[bool] = False) -> str:
        """
        :class:`str`: Gets the lyrics of a song.

        Parameters
        ----------
        song : The name of the song. :class:`str`
        simple : Wheter the lyrics should be simple. Optional[:class:`bool`]
        """
        response = await HTTPClient().get(f"lyrics?song={song}&simple={simple}")

        if response.status == 200:
            json = await response.json()
            return json["data"]["lyrics"]

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def song_info(cls, song: str) -> SongInfo:
        """
        :class:`str`: Gets the information of a song.

        Parameters
        ----------
        song : The name of the song. :class:`str`
        """
        response = await HTTPClient().get(f"songinfo?song={song}")

        if response.status == 200:
            json = await response.json()
            return SongInfo(json)

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def minecraft_status(cls, host: str, port: Optional[str] = None) -> str:
        """
        :class:`str`: Gets the status of a minecraft server.

        Parameters
        ----------
        host : The hostname of the server. Required[:class:`str`]
        port : The port of the server. Optional[:class:`str`]
        """
        response = await HTTPClient().get(f"mcstatus?host={host}&port={port}")

        if response.status == 200:
            json = await response.json()
            return json  # Don't know what it returns

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def bored(cls) -> str:
        """
        :class:`str`: Gets a bored fact.
        """
        response = await HTTPClient().get("bored")

        if response.status == 200:
            json = await response.json()
            return json["data"]["activity"]

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def numberfact(cls, number: int) -> str:
        """
        :class:`str`: Gets a fact about a number.

        Parameters
        ----------
        number : The number to get a fact about. :class:`int`
        """
        response = await HTTPClient().get(f"numberfact?number={number}")

        if response.status == 200:
            json = await response.json()
            return json["data"]

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")



from setuptools import setup

from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

requirements = []
with open("requirements.txt") as f:
    requirements = f.read().splitlines()

VERSION = "0.0.9"

setup(
    name="dhravyapy",
    author="27Saumya",
    url="https://github.com/27Saumya/dhravyapy",
    download_url="https://github.com/27Saumya/dhravyapy/archive/v{}.tar.gz".format(VERSION),
    version=VERSION,
    packages=["dhravyapy"],
    license="MIT",
    description="An asynchronous wrapper for interacting with the Dhravya API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=requirements,
    python_requires=">=3.8.0",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



"""
MIT License

Copyright (c) 2022 27Saumya

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from .http import HTTPClient
from typing import *
from .errors import *
from .assets import Meme, TruthOrDare, GeneralImage


class Fun:
    """
    The class to get fun related data from the API
    """

    @classmethod
    async def eightball(cls, *, simple: Optional[bool] = False) -> str:
        """
        :class:`str`: Gets a random 8ball response.

        Parameters
        ----------
        simple : Wheter the response should be simple. Optional[:class:`bool`]
        """
        response = await HTTPClient().get(f"fun/8ball?simple={simple}")

        if response.status == 200:
            json = await response.json()
            return json["data"]["answer"]

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def meme(cls, *, topic: Optional[str] = None) -> Union[Meme, GeneralImage]:
        """
        :class:`Meme`: Gets a meme from the API.

        Parameters
        ----------
        topic : :class:`str`
        """
        if topic:
            response = await HTTPClient().get(f"meme/{topic}")

            if response.status == 200:
                json = await response.json()
                return Meme(json)

            elif response.status == 422:
                raise ValidationError("Recieved an invalid input")

            elif response.status == 500:
                raise HTTPException("Internal Server Error")

            else:
                raise HTTPException(f"HTTP Error: {response.status}")

        else:
            response = await HTTPClient().get("meme")

            if response.status == 200:
                return GeneralImage(await response.read())

            elif response.status == 422:
                raise ValidationError("Recieved an invalid input")

            else:
                raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def wyr(cls, simple: Optional[bool] = False) -> List[str]:
        """
        :class:`str`: Gets a random "Would You Rather" question.

        Parameters
        ----------
        simple : Wheter the question should be simple. Optional[:class:`bool`]
        """
        response = await HTTPClient().get(f"wyr?simple={simple}")

        if response.status == 200:
            json = await response.json()
            return json["data"]["Would You Rather"]

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def truthordare(cls, simple: Optional[bool] = False) -> TruthOrDare:
        """
        :class:`str`: Gets a random truth or dare.

        Parameters
        ----------
        simple : Wheter the question should be simple. Optional[:class:`bool`]
        """
        response = await HTTPClient().get(f"truthordare?simple={simple}")

        if response.status == 200:
            json = await response.json()
            return TruthOrDare(json)

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def roast(cls, simple: Optional[bool] = False) -> str:
        """
        :class:`str`: Gets a random roast.

        Parameters
        ----------
        simple : Wheter the roast should be simple. Optional[:class:`bool`]
        """
        response = await HTTPClient().get(f"roast?simple={simple}")

        if response.status == 200:
            json = await response.json()
            return json["data"]["Roast"]

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def truth(cls, simple: Optional[bool] = False) -> str:
        """
        :class:`str`: Gets a random truth.

        Parameters
        ----------
        simple : Wheter the truth should be simple. Optional[:class:`bool`]

        Note::
        ------
            This is a shortcut for :meth:`Fun().truthordare().truth`.
        """
        response = await HTTPClient().get(f"truth?simple={simple}")

        if response.status == 200:
            json = await response.json()
            return json["data"]["Truth"]

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def dare(cls, simple: Optional[bool] = False) -> str:
        """
        :class:`str`: Gets a random dare.

        Parameters
        ----------
        simple : Wheter the dare should be simple. Optional[:class:`bool`]

        Note::
        ------
            This is a shortcut for :meth:`Fun().truthordare().dare`.
        """
        response = await HTTPClient().get(f"dare?simple={simple}")

        if response.status == 200:
            json = await response.json()
            return json["data"]["Dare"]

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def joke(cls, simple: Optional[bool] = False) -> str:
        """
        :class:`str`: Gets a random joke.

        Parameters
        ----------
        simple : Wheter the joke should be simple. Optional[:class:`bool`]
        """
        response = await HTTPClient().get(f"joke?simple={simple}")

        if response.status == 200:
            json = await response.json()
            return json["data"]["Joke"]

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")

    @classmethod
    async def neverhaveiever(cls, simple: Optional[bool] = False) -> str:
        """
        :class:`str`: Gets a random "Never Have I Ever" question.

        Parameters
        ----------
        simple : Wheter the question should be simple. Optional[:class:`bool`]
        """
        response = await HTTPClient().get(f"neverhaveiever?simple={simple}")

        if response.status == 200:
            json = await response.json()
            return json["data"]["Topic"]

        elif response.status == 422:
            raise ValidationError("Recieved an invalid input")

        else:
            raise HTTPException(f"HTTP Error: {response.status}")
